require("gui_creation")

--A file that contains the on_event scripting for the mod's gui.

script.on_event({defines.events.on_gui_click},
function(event)
    local player = game.players[event.player_index]
    local cGui = player.gui.center
    local clicked = event.element

    if not string.find(clicked.name, "ygm") then --has nothing to do with this mod
        return
    end

    if event.button == defines.mouse_button_type.right then --if they right click the upper left button
        player.print("Resetting UI...")
        nukeAndRegenUI(player)
        return
    end

    if clicked.name == "ygm-open-mail-list-button" then
        if player.gui.left["ygm-open-mail-list-button"].sprite == "ygm-new-mail-sprite" then
            player.gui.left["ygm-open-mail-list-button"].sprite = "ygm-mail-sprite" --set the mail icon to indicate that the mail has been checked
        end
        player.gui.center["ygm-mail-list-frame"].style.visible = true
    elseif clicked.name == "ygm-close-parent-button" then
        if clicked.parent.name ~= "ygm-mail-list-frame" then --don't destroy the mail list frame
            clicked.parent.destroy()
            cGui["ygm-mail-list-frame"].style.visible = true
        else
            clicked.parent.style.visible = false
        end
    elseif clicked.name == "ygm-open-mail-spritebutton" then
        if global.player_mail[player.index][cGui["ygm-mail-list-frame"]["ygm-mail-list-dropdown"].selected_index] then
            clicked.parent.parent.style.visible = false
            makeReadMailGUI(cGui)
        else
            player.print("There is no mail to read.")
        end
    elseif clicked.name == "ygm-create-mail-spritebutton" then
        clicked.parent.parent.style.visible = false
        makeNewMailGui(cGui)
    elseif clicked.name == "ygm-delete-mail-spritebutton" then
        if event.control then
            --do mail delete
            deleteCurrentMail(player)
        else
            player.print("Hold Ctrl to delete mail.")
        end
    elseif clicked.name == "ygm-close-parent-parent-button" then
        clicked.parent.parent.destroy()
        player.gui.center["ygm-mail-list-frame"].style.visible = true
    elseif clicked.name == "ygm-send-mail-button" then
        local toDropDown = cGui["ygm-new-mail-frame"]["ygm-to-flow"]["ygm-to-dropdown"]
        local destination = toDropDown.items[toDropDown.selected_index]
        local subject = cGui["ygm-new-mail-frame"]["ygm-subject-flow"]["ygm-subject-textfield"].text
        local message = cGui["ygm-new-mail-frame"]["ygm-mail-text-box"].text

        sendMail(destination, subject, message, player.name)

        cGui["ygm-new-mail-frame"].destroy() --close the new mail frame
    elseif clicked.name == "ygm-reply-button" then
        local mail = global.player_mail[player.index][cGui["ygm-mail-list-frame"]["ygm-mail-list-dropdown"].selected_index]

        makeReplyGui(cGui, mail.subject, player.name, mail.player)
        clicked.parent.parent.destroy()
    elseif clicked.name == "ygm-send-reply-button" then
        local mail = global.player_mail[player.index][cGui["ygm-mail-list-frame"]["ygm-mail-list-dropdown"].selected_index]
        local replyMessage = cGui["ygm-reply-frame"]["ygm-reply-textbox"].text
        local newSubject = mail.subject

        if not mail.subject:find("Re:") then -- Only add Re: once
            newSubject = "Re: " .. mail.subject
        end

        if game.players[mail.player] then --only send reply if the mail actually came from someone
            deleteCurrentMail(player) -- Delete the mail they just replied to
            sendMail(mail.player, newSubject, replyMessage, player.name)
        else
            player.print("Cannot send reply to non-existent player.")
        end

        cGui["ygm-reply-frame"].destroy()
    elseif clicked.name == "ygm-save-mail-button" then
        player.print("Writing current mail to disk...")
        local mail = global.player_mail[player.index][cGui["ygm-mail-list-frame"]["ygm-mail-list-dropdown"].selected_index]
        -- construct the saved mail
        local write = "" .. mail.subject .. " | Wrote at game tick: " .. game.tick .. "\n"
        write = write .. "From: " .. mail.player .. "\n"
        write = write .. "Begin Mail Body ------------------\n"
        write = write .. mail.message .. "\n"
        write = write .. "End Mail Body --------------------"

        game.write_file("saved_mail_" .. game.tick .. ".txt", write, false, player.index) --write the file
    end
end --ends function
)

-- Sends the mail to to player, using subject and message provided
function sendMail(dest, subject, message, from)
    local player = game.players[dest]

    --fix the subject
    if subject:len() == 0 then
        subject = "No subject."
    elseif subject:len() > 40 then --truncate subjects longer than 40 chars
        subject = subject:sub(1,40) .. "..."
    end

    --long message spam protect, truncate messages longer than 1500 chars
    if message:len() >= 1500 then
        message = message:sub(1,1500) .. "[Anti-Spam Truncation]"
    end

    -- construct the mail
    local mail = {
        player = from,
        subject = subject,
        message = message
    }
    table.insert(global.player_mail[player.index], mail) --send the mail
    getMailAsStrings(player.gui.center["ygm-mail-list-frame"]["ygm-mail-list-dropdown"]) --fix the player's dropdown
    player.print("You've got mail!")
    player.gui.left["ygm-open-mail-list-button"].sprite = "ygm-new-mail-sprite"
    log("Player " .. dest .. " was sent mail by " .. from .. ", with subject: " .. subject)

    game.players[from].print("Sent.")
end

-- Deletes the currently selected mail
function deleteCurrentMail(player)
    local dropDown = player.gui.center["ygm-mail-list-frame"]["ygm-mail-list-dropdown"]
    if not global.player_mail[player.index][dropDown.selected_index] then
        player.print("There is no mail to delete.")
        return
    end

    --remove the mail from the table
    table.remove(global.player_mail[player.index],dropDown.selected_index)

    local itemsList = dropDown.items
    table.remove(itemsList, dropDown.selected_index)
    dropDown.items = itemsList
    dropDown.selected_index = #dropDown.items --done to refresh the dropdown
end
