--A file that holds all the functions this mod uses for gui creation.

function nukeAndRegenUI(player)
    local lGui = player.gui.left
    local cGui = player.gui.center

    --Clear the existing mod's GUI
    if cGui["ygm-mail-list-frame"] then
        cGui["ygm-mail-list-frame"].destroy()
    end

    if cGui["ygm-read-mail-frame"] then
        cGui["ygm-read-mail-frame"].destroy()
    end

    if cGui["ygm-new-mail-frame"] then
        cGui["ygm-new-mail-frame"].destroy()
    end

    if cGui["ygm-reply-frame"] then
        cGui["ygm-reply-frame"].destroy()
    end


    if lGui["ygm-open-mail-list-button"] then
        lGui["ygm-open-mail-list-button"].destroy()
    end

    --add the button to open the player's mail
    lGui.add{
        type = "sprite-button",
        tooltip = "Click to open your mail.",
        name = "ygm-open-mail-list-button",
        sprite = "ygm-mail-sprite",
        style = "ygm_small_buttons"
    }

    constructMailList(cGui)
end


function constructMailList(gui)
    --Add the base frame
    local frame = gui.add{
        name = "ygm-mail-list-frame",
        type = "frame",
        caption = "Mail Inbox",
        direction = "vertical"
    }
    frame.style.visible = false

    local dropdown = frame.add{
        name = "ygm-mail-list-dropdown",
        type = "drop-down",
    }
    getMailAsStrings(dropdown)

    local flow = frame.add{
        name = "ygm-mail-list-actions-flow",
        type = "flow",
        direction = "horizontal"
    }

    flow.add{
        name = "ygm-open-mail-spritebutton",
        type = "sprite-button",
        style = "ygm_buttons",
        sprite = "ygm-open-mail-sprite",
        tooltip = "Open the selected mail."
    }

    flow.add{
        name = "ygm-create-mail-spritebutton",
        type = "sprite-button",
        style = "ygm_buttons",
        sprite = "ygm-create-mail-sprite",
        tooltip = "Create a new mail."
    }

    flow.add{
        name = "ygm-delete-mail-spritebutton",
        type = "sprite-button",
        style = "ygm_buttons",
        sprite = "ygm-delete-sprite",
        tooltip = "Delete the selected mail."
    }

    frame.add{
        name = "ygm-close-parent-button",
        type = "button",
        caption = "Close",
    }
end

--Sets the dropdown's items to be a string table of the subjects of the player's mail
function getMailAsStrings(dropdown)
    local strings = {}
    local player = dropdown.parent.gui.player
    for index, mail in pairs(global.player_mail[player.index]) do --get list of mail subjects
        local displaySubject = mail.subject .. " | " .. mail.player
        table.insert(strings, displaySubject)
    end
    dropdown.items = strings
    dropdown.selected_index = #strings
end

-- Returns a table of player usernames
function getPlayersAsStrings()
    local strings = {}
    for index, player in pairs(game.players) do
        table.insert(strings, player.name)
    end
    return strings
end

-- Makes the gui for reading a mail.
function makeReadMailGUI(gui)
    local player = game.players[gui.player_index]
    local mail = global.player_mail[gui.player_index][gui["ygm-mail-list-frame"]["ygm-mail-list-dropdown"].selected_index]
    local mailFrame= gui.add{
        name = "ygm-read-mail-frame",
        type = "frame",
        direction = "horizontal",
        caption = mail.subject .. " - From: " .. mail.player
    }

    mailFrame.style.minimal_width = 700
    mailFrame.style.maximal_height= 600
    mailFrame.style.minimal_height = 600
    mailFrame.style.maximal_width = 700

    local textBox= mailFrame.add{
        type = "text-box",
        name = "ygm-mail-text-box",
    }

    textBox.word_wrap = true
    textBox.style.minimal_width = 600
    textBox.style.maximal_height= 500
    textBox.style.minimal_height = 500
    textBox.style.maximal_width = 600
    textBox.text = global.player_mail[gui.player_index][gui["ygm-mail-list-frame"]["ygm-mail-list-dropdown"].selected_index].message or "No message."
    textBox.enabled = false --disallow editing

    local replyClose = mailFrame.add{
        name = "ygm-reply-close-flow",
        type = "flow",
        direction = "vertical"
    }
    replyClose.add{
        name = "ygm-reply-button",
        type = "button",
        caption = "Reply",
        tooltip = "Reply to sender."
    }

    replyClose.add{
        name = "ygm-save-mail-button",
        type = "button",
        caption = "Save",
        tooltip = "Save this mail to disk."
    }

    replyClose.add{
        name = "ygm-close-parent-parent-button",
        type = "button",
        caption = "Close",
    }

    gui["ygm-mail-list-frame"].style.visible = false --hide mail list
end

-- Makes the gui for making a new mail
function makeNewMailGui(gui)
    local player = game.players[gui.player_index]
    local newMailFrame= gui.add{
        name = "ygm-new-mail-frame",
        type = "frame",
        direction = "vertical",
        caption = "New Mail"
    }
    newMailFrame.style.minimal_width = 550
    newMailFrame.style.maximal_height= 600
    newMailFrame.style.minimal_height = 600
    newMailFrame.style.maximal_width = 550

    -- To: line
    local toFlow = newMailFrame.add{
        name = "ygm-to-flow",
        type = "flow",
        direction = "horizontal"
    }
    toFlow.add{
        name = "ygm-to-label",
        type= "label",
        caption = "To: "
    }
    local toDropDown = toFlow.add{
        name = "ygm-to-dropdown",
        type= "drop-down"
    }
    toDropDown.items = getPlayersAsStrings() -- Get a list of player usernames
    toDropDown.selected_index = 1

    -- Subject line
    local subjectFlow = newMailFrame.add{
        name = "ygm-subject-flow",
        type = "flow",
        direction = "horizontal"
    }
    subjectFlow.add{
        name = "ygm-subject-label",
        type= "label",
        caption = "Subject: "
    }
    subjectFlow.add{
        name = "ygm-subject-textfield",
        type = "textfield"
    }
    -- Mail body
    local mailTextBox = newMailFrame.add{
        name = "ygm-mail-text-box",
        type = "text-box",
    }
    mailTextBox.word_wrap = true
    mailTextBox.style.minimal_width = 500
    mailTextBox.style.maximal_height= 400
    mailTextBox.style.minimal_height = 400
    mailTextBox.style.maximal_width = 500
    mailTextBox.text ="New message"

    --send/cancel buttons
    local sendFlow = newMailFrame.add{
        name = "ygm-send-flow",
        type = "flow",
        direction = "horizontal",
    }
    sendFlow.add{
        name = "ygm-send-mail-button",
        type = "sprite-button",
        tooltip = "Send the mail to the player.",
        sprite = "ygm-confirm-sprite",
        style = "ygm_buttons"
    }
    sendFlow.add{
        name = "ygm-close-parent-parent-button",
        type= "button",
        caption = "Close",
    }
end

--Makes the gui for replying to a mail.
function makeReplyGui(gui, subject, from, to)
    local newSubject = subject

    if not subject:find("Re:") then -- Only add Re: once
        newSubject = "Re: " .. subject
    end
    local frame = gui.add{
        name = "ygm-reply-frame",
        type = "frame",
        direction = "vertical",
        caption = newSubject
    }
    frame.style.minimal_width = 550
    frame.style.maximal_height= 600
    frame.style.minimal_height = 600
    frame.style.maximal_width = 550

    frame.add{
        name = "ygm-reply-to-label",
        caption = "Replying to " .. to,
        type = "label"
    }

    local textBox = frame.add{
        name = "ygm-reply-textbox",
        type = "text-box"
    }

    textBox.word_wrap = true
    textBox.style.minimal_width = 500
    textBox.style.maximal_height= 400
    textBox.style.minimal_height = 400
    textBox.style.maximal_width = 500
    textBox.text ="Write your reply here."

    frame.add{
        name = "ygm-send-reply-button",
        type = "sprite-button",
        sprite = "ygm-confirm-sprite",
        tooltip = "Send your reply.",
        style = "ygm_buttons"
    }

    frame.add{
        name = "ygm-close-parent-button",
        type = "button",
        caption = "Close"
    }
end
